# main.py

import os
import sys
import cv2
import lib
import shutil
from concurrent.futures import ThreadPoolExecutor
import argparse
from tqdm import tqdm

NUM_THREADS = 16

def process_image(image_directory, output_directory, filename, threshold, algorithm):
    image_path = os.path.join(image_directory, filename)
    img = cv2.imread(image_path)

    # Check if the image is blurred using the specified algorithm
    blurry, blurryness = lib.is_image_blurry(img, algorithm=algorithm, threshold=threshold)

    if blurry:
        # Move the blurred image to the output directory
        output_path = os.path.join(output_directory, filename)
        shutil.move(image_path, output_path)
        tqdm.write(f"Moved blurred image: {filename} to {output_directory}, blurriness: {blurryness}")

def main():
    parser = argparse.ArgumentParser(description='Detect and move blurred images.')
    parser.add_argument('image_directory', type=str, help='Directory containing images')
    parser.add_argument('-t', '--threshold', type=int, default=600, help='Blurry detection threshold')
    parser.add_argument('-a', '--algorithm', choices=['laplacian', 'sobel', 'fourier'], default='laplacian', help='Blur detection algorithm')

    args = parser.parse_args()

    image_directory = args.image_directory
    output_directory = os.path.join(image_directory, "blurred")

    # Create output directory if it doesn't exist
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)

    # Get the list of image filenames
    image_filenames = [filename for filename in os.listdir(image_directory) if filename.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp'))]

    # Create a ThreadPoolExecutor with 16 worker threads
    with ThreadPoolExecutor(max_workers=NUM_THREADS) as executor:
        # Use tqdm to create a progress bar
        with tqdm(total=len(image_filenames), desc="Processing images") as pbar:
            # Iterate through images in the specified directory and submit tasks to the executor
            futures = []
            for filename in image_filenames:
                future = executor.submit(process_image, image_directory, output_directory, filename, args.threshold, args.algorithm)
                future.add_done_callback(lambda p: pbar.update())
                futures.append(future)

            # Wait for all tasks to complete
            for future in futures:
                future.result()

if __name__ == "__main__":
    main()
